// NameColouriser.java
//
// (c) 1999-2001 PAL Development Core Team
//
// This package may be distributed under the
// terms of the Lesser GNU General Public License (LGPL)

package pal.gui;

/**
 * Title:        NameColouriser.java
 * Description:  A means for mapping names to colours
 * @author      Matthew Goode
 * @version 1.0
 */
import java.util.*;
import java.awt.*;
import pal.misc.*;
public final class NameColouriser implements java.io.Serializable {
	
	Hashtable colourMappings_ = new Hashtable();
	
	public NameColouriser() { }
	
	public NameColouriser(String name, Color colour) {
		addMapping(name,colour);
	}
	
	public NameColouriser(Identifier name, Color colour) {
		addMapping(name,colour);
	}
	
	public void addMapping(String name, Color colour) {
		colourMappings_.put(name,colour);
	}
	
	public void addMapping(Identifier id, Color colour) {
		if(id!=null&&id.getName()!=null) {
			colourMappings_.put(id.getName(),colour);
		}
	}
	
	/**
	 * @param names Names
	 * @param colours associated colours
	 * @note assumes parallel arrays
	 */
	public void addMappings(String[] names, Color[] colours) {
		for(int i = 0 ; i < names.length ; i++) {
			colourMappings_.put(names[i],colours[i]);
		}
	}

	public Color getColour(String name, Color defaultColour) {
		if(name==null||!colourMappings_.containsKey(name)) {
			return defaultColour;
		}
		return (Color)colourMappings_.get(name);
	}
	
	public Color getColour(Identifier id, Color defaultColour) {
		if(id==null) {
			return defaultColour;
		}
		return getColour(id.getName(),defaultColour);
	}

}
