// SubstitutionModel.java
//
// (c) 1999-2001 PAL Development Core Team
//
// This package may be distributed under the
// terms of the Lesser GNU General Public License (LGPL)


package pal.substmodel;

import pal.misc.*;
import pal.math.*;
import pal.datatype.*;

import java.io.*;


/**
 * <b>model of sequence substitution (rate matrix + rate variation)</b>.
 * provides a convenient interface for the computation of transition probabilities
 *
 * @version $Id: SubstitutionModel.java,v 1.18 2001/10/10 07:50:02 matt Exp $
 *
 * @author Alexei Drummond
 */
public interface SubstitutionModel extends Parameterized, Report, java.io.Serializable {

	/**
	 * @return the rate matrix
	 */
	RateMatrix getRateMatrix();

	/**
	 * @return the rate distribution of this substitution model.
	 */
	RateDistribution getRateDistribution();

	/**
	 * set distance and corresponding computation transition probabilities
	 *
	 * @param k distance
	 */
	void setDistance(double k);

	/**
	 * set times and corresponding computation transition probabilities
	 *
	 * @param start start time
	 * @param end end time
	 */
 //	void setTime(double start, double end);

	/**
	 * get transition probability for the preselected model and
	 * the previously specified distance
	 *
	 * @param r rate category
	 * @param i start state
	 * @param j end state
	 *
	 * @return transition probability
	 */
	double transProb(int r, int i, int j);

	/** Returns the dimension of this model.
			@deprecated
	 */
	int getDimension();

	DataType getDataType();

	/** Add a PalObjectListener to be notified of changes to the model.
	 *  Only the parametersChanged method will generally be called
	 */
	void addPalObjectListener(PalObjectListener pol);
	void removePalObjectListener(PalObjectListener pol);

	/** A non shallow implementation of clone() */
	Object clone();

}
